package com.yx.face.service.job;

import cn.hutool.core.date.DateUtil;
import com.yx.face.dao.IssueDeviceDao;
import com.yx.face.dao.IssueDeviceItemDao;
import com.yx.face.model.entity.issue.IssueDevice;
import com.yx.face.model.entity.issue.IssueDeviceItem;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.context.annotation.Configuration;
import org.springframework.scheduling.annotation.EnableScheduling;
import org.springframework.scheduling.annotation.Scheduled;
import tk.mybatis.mapper.entity.Example;
import tk.mybatis.mapper.weekend.WeekendSqls;

import javax.annotation.Resource;
import java.util.Date;
import java.util.List;
import java.util.stream.Collectors;

/**
 * @author shisl
 * @package com.yx.face.service.job
 * @class FkjQrcodeTask
 * @date 2023/7/29 下午9:41
 * @description 删除过期的下发记录job
 */
@Configuration
@EnableScheduling
@Slf4j
public class ClearIssueDeviceTask {

    @Resource
    private IssueDeviceDao issueDeviceDao;

    @Resource
    private IssueDeviceItemDao issueDeviceItemDao;

    @Value("${hhface.issue.holdDays}")
    private Integer issueHoldDays;

    /**
     * 删除过期的下发记录
     * 每天凌晨1点
     */
    @Scheduled(cron = "0 0 1 * * ?", zone = "Asia/Shanghai")
    public void opDeviceOffline() {
        log.info("--->开始执行删除过期的下发记录");
        Date dateTime = DateUtil.offsetDay(new Date(), -issueHoldDays);
        List<IssueDevice> issueDevices = issueDeviceDao.selectByExample(Example.builder(IssueDevice.class).select("id")
                .where(WeekendSqls.<IssueDevice>custom().andLessThan(IssueDevice::getCreateTime, dateTime)).build());
        if (issueDevices == null || issueDevices.size() == 0) {
            log.info("--->结束执行删除过期的下发记录，执行数{}", 0);
            return;
        }
        int delCount = issueDeviceDao.deleteByExample(Example.builder(IssueDevice.class)
                .where(WeekendSqls.<IssueDevice>custom().andLessThan(IssueDevice::getCreateTime, dateTime)).build());

        List<Long> issueDeviceIds = issueDevices.stream().map(IssueDevice::getId).collect(Collectors.toList());
        issueDeviceItemDao.deleteByExample(Example.builder(IssueDeviceItem.class)
                .where(WeekendSqls.<IssueDeviceItem>custom().andIn(IssueDeviceItem::getIssueDeviceId, issueDeviceIds)).build());
        log.info("--->结束执行删除过期的下发记录，执行数{}", delCount);
    }

}
